@@ -12,6 +12,7 @@ from aiortc.rtcrtpreceiver import RemoteStreamTrack
 from queue import Queue
 from gspeerconnection.gsmediarecorder import GSMediaRecorder
 from threading import Thread
+import time
 
 relay = MediaRelay()
 
@@ -43,106 +44,6 @@ class GSCOMMAND(StrEnum):
     IRCUT = 'ircut'
 
 
-class GSPeerConnectionWatcher:
-
-    def __init__(self):
-        self.sio = None
-        self.oncommand = None
-
-    async def sendCommand(self, command: GSCOMMAND, value):
-        await self.sio.emit("command", {"type": command, "value": value})
-
-    @classmethod
-    async def create(cls, connectionWatcher, gsdbs, target, onframe=None, onmessage=None, ontrack=None,
-                     ontrackended=None):
-        self = connectionWatcher
-        self.rtconfiList = []
-        self.sio = socketio.AsyncClient()
-        self.gsdbs = gsdbs
-        self.onframe = onframe
-        self.target = target
-        self.onmessage = onmessage
-        self.ontrack = ontrack
-        self.ontrackended = ontrackended
-        self.logger = logging.getLogger(__name__)
-
-        if self.gsdbs.credentials["stunenable"]:
-            self.rtconfiList.append(RTCIceServer(self.gsdbs.credentials["stunserver"]))
-        if self.gsdbs.credentials["turnenable"]:
-            self.rtconfiList.append(RTCIceServer(self.gsdbs.credentials["turnserver"],
-                                                 self.gsdbs.credentials["turnuser"],
-                                                 self.gsdbs.credentials["turnpw"]))
-
-        @self.sio.event
-        async def connect():
-            self.logger.info('connection established')
-
-        @self.sio.event
-        async def joined(id):
-            self.logger.info("joined")
-
-        @self.sio.event
-        async def broadcaster(id):
-            if len(self.rtconfiList) > 0:
-                self.peerConnections = RTCPeerConnection(configuration=RTCConfiguration(self.rtconfiList))
-            else:
-                self.peerConnections = RTCPeerConnection()
-
-            self.peerConnections.addTransceiver('video', direction='recvonly')
-
-            @self.peerConnections.on("iceconnectionstatechange")
-            async def on_iceconnectionstatechange():
-                if self.peerConnections.iceConnectionState == "complete":
-                    pass
-                if self.peerConnections.iceConnectionState == "failed":
-                    await self.peerConnections.close()
-
-            @self.peerConnections.on("track")
-            async def on_track(track):
-                if track.kind == "video":
-                    if self.ontrack is not None:
-                        self.logger.info("on track received.Recording started")
-                        await self.ontrack(self.gsdbs, track, self.target)
-                    if self.onframe is not None:
-                        self.logger.info("on track received. onframe started")
-                        gsmediaconsumer = GSMediaConsumer(self.gsdbs, self.target, self.onframe)
-                        gsmediaconsumer.addTrack(track)
-                        await gsmediaconsumer.start()
-
-            await self.peerConnections.setLocalDescription(await self.peerConnections.createOffer())
-            await self.sio.emit("watcher",
-                                {"target": self.target,
-                                 "sdp": {"type": self.peerConnections.localDescription.type,
-                                         "sdp": self.peerConnections.localDescription.sdp}})
-
-        @self.sio.event
-        async def answer(id, description):
-            if isinstance(description, dict):
-                desc = type('new_dict', (object,), description)
-            else:
-                desc = type('new_dict', (object,), ast.literal_eval(description))
-            await self.peerConnections.setRemoteDescription(desc)
-
-        @self.sio.event
-        async def disconnect(id, target):
-            self.logger.info("Socket connection list")
-            await self.ontrackended(self.gsdbs, target)
-
-        @self.sio.event
-        async def disconnectBroadcaster(id, target):
-            self.logger.info("Camera connection list")
-            await self.ontrackended(self.gsdbs, target)
-
-        if "localhost" in self.gsdbs.credentials["signalserver"]:
-            connectURL = f'{self.gsdbs.credentials["signalserver"]}:{str(self.gsdbs.credentials["signalport"])}'
-        else:
-            connectURL = self.gsdbs.credentials["signalserver"]
-
-        await self.sio.connect(
-            f'{connectURL}?gssession={self.gsdbs.cookiejar.get("session")}.{self.gsdbs.cookiejar.get("signature")}{self.gsdbs.credentials["cnode"]}&target={self.target}')
-        await self.sio.wait()
-
-
 class GSPeerConnectionWatcherNew:
 
     def __init__(self):
@@ -152,9 +53,27 @@ class GSPeerConnectionWatcherNew:
     async def sendCommand(self, command: GSCOMMAND, value):
         await self.sio.emit("command", {"type": command, "value": value})
 
+    async def process_frames(self, frame_queue, framerate):
+        self.logger.warning(f"starting process frames")
+        start_time = time.time()
+        frame_count = 0
+        fps = framerate
+        while True:
+            frame_count += 1
+            elapsed_time = time.time() - start_time
+            if elapsed_time > 1 / fps:
+                current_fps = frame_count / elapsed_time
+                print(f'Async Loop FPS: {elapsed_time:.3f}')
+                start_time = time.time()
+                frame_count = 0
+                frame_data = await frame_queue.get()
+                self.onframe(self.gsdbs, self.target, frame_data, 0)
+                frame_queue.task_done()
+            await asyncio.sleep(0)
+
     @classmethod
-    async def create(cls, connectionWatcher, gsdbs, target, onframe=None, onmessage=None, ontrack=None,
-                     ontrackended=None, signalingserver=None):
+    async def create(cls, connectionWatcher, gsdbs, target, onframe=None, onmessage=None, ontrack=None, sync=True,
+                     framerate=15, ontrackended=None, signalingserver=None):
         self = connectionWatcher
         self.rtconfiList = []
         self.sio = socketio.AsyncClient()
@@ -167,6 +86,8 @@ class GSPeerConnectionWatcherNew:
         self.ontrack = ontrack
         self.ontrackended = ontrackended
         self.signalingserver = signalingserver
+        self.sync = sync
+        self.framerate = framerate
         self.logger = logging.getLogger(__name__)
 
         if self.gsdbs.credentials["stunenable"]:
@@ -218,7 +139,11 @@ class GSPeerConnectionWatcherNew:
                         await self.ontrack(self.gsdbs, track, self.target)
                     if self.onframe is not None:
                         self.logger.info("on track received. onframe started")
-                        gsmediaconsumer = GSMediaConsumer(self.gsdbs, self.target, self.onframe)
+                        if self.sync:
+                            gsmediaconsumer = GSMediaConsumer(self.gsdbs, self.target, self.onframe, self.framerate)
+                        else:
+                            gsmediaconsumer = GSMediaConsumerAsync(self.gsdbs, self.target, self.onframe,
+                                                                   self.frame_queue)
                         gsmediaconsumer.addTrack(track)
                         await gsmediaconsumer.start()
 
@@ -251,9 +176,13 @@ class GSPeerConnectionWatcherNew:
                             await self.ontrack(self.gsdbs, track, self.target)
                         if self.onframe is not None:
                             self.logger.info("on track received. onframe started")
-                            self.gsmediaconsumer = GSMediaConsumer(self.gsdbs, self.target, self.onframe)
-                            self.gsmediaconsumer.addTrack(track)
-                            await self.gsmediaconsumer.start()
+                            if self.sync:
+                                gsmediaconsumer = GSMediaConsumer(self.gsdbs, self.target, self.onframe, self.framerate)
+                            else:
+                                gsmediaconsumer = GSMediaConsumerAsync(self.gsdbs, self.target, self.onframe,
+                                                                       self.frame_queue)
+                            gsmediaconsumer.addTrack(track)
+                            await gsmediaconsumer.start()
 
                 await self.peerConnections.setRemoteDescription(offer)
                 await self.peerConnections.setLocalDescription(await self.peerConnections.createAnswer())
@@ -300,41 +229,56 @@ class GSPeerConnectionWatcherNew:
 
         await self.sio.connect(
             f'{connectURL}?gssession={self.gsdbs.cookiejar.get("session")}.{self.gsdbs.cookiejar.get("signature")}{self.gsdbs.credentials["cnode"]}&target={self.target}&terminator=true')
-        await self.sio.wait()
+
+        if self.sync:
+            await self.sio.wait()
+        else:
+            self.frame_queue = asyncio.Queue(3)
+            process_task = asyncio.create_task(self.process_frames(self.frame_queue, self.framerate))
+            return await asyncio.gather(self.sio.wait(), process_task)
 
 
 class FrameBufferThread(Thread):
-    def __init__(self, gsdbs, target, onframe):
+    def __init__(self, gsdbs, target, onframe, framerate):
         super(FrameBufferThread, self).__init__()
         self.gsdbs = gsdbs
         self.logger = logging.getLogger(__name__)
         self.logger.debug("Starting Buffer")
         self.target = target
         self.onframe = onframe
-        self.framerate = 0
+        self.framerate = framerate
         self.nal_queue = Queue(3)
 
-    def write(self, frame, framerate=0):
-        self.framerate = framerate
+    def write(self, frame):
         if not self.nal_queue.full():
             self.nal_queue.put(frame)
 
     def run(self):
+        start_time = time.time()
+        frame_count = 0
+        fps = self.framerate
         while True:
-            buf = self.nal_queue.get()
-            if buf is not None or buf:
-                self.onframe(self.gsdbs, self.target, buf, self.framerate)
+            frame_count += 1
+            elapsed_time = time.time() - start_time
+            if elapsed_time > 1 / fps:
+                buf = self.nal_queue.get()
+                current_fps = frame_count / elapsed_time
+                print(f'Sync Loop FPS: {elapsed_time:.2f}')
+                start_time = time.time()
+                frame_count = 0
+                if buf is not None or buf:
+                    self.onframe(self.gsdbs, self.target, buf, self.framerate)
 
 
 class GSMediaConsumer:
 
-    def __init__(self, gsdbs, target, onframe):
+    def __init__(self, gsdbs, target, onframe, framerate):
         self.__tracks = {}
         self.gsdbs = gsdbs
         self.target = target
         self.onframe = onframe
         self.nal_queue = Queue(3)
-        self.broadcastthread = FrameBufferThread(self.gsdbs, self.target, self.onframe)
+        self.broadcastthread = FrameBufferThread(self.gsdbs, self.target, self.onframe, framerate)
         self.broadcastthread.start()
 
     def addTrack(self, track):
@@ -347,10 +291,6 @@ class GSMediaConsumer:
         """
         for track, task in self.__tracks.items():
             if task is None:
-                # self.startProcessing(task)
-                # _thread = threading.Thread(target=self.startProcessing,
-                #                            args=(track,))
-                # _thread.start()
                 self.__tracks[track] = asyncio.ensure_future(
                     self.stream_consume(track, self.gsdbs, self.target, self.onframe))
 
@@ -372,44 +312,14 @@ class GSMediaConsumer:
         self.__tracks = {}
 
 
-class FrameBufferThreadAsync():
-    def __init__(self, gsdbs, target, onframe):
-        self.gsdbs = gsdbs
-        self.logger = logging.getLogger(__name__)
-        self.logger.debug("Starting Buffer")
-        self.target = target
-        self.onframe = onframe
-        self.framerate = 0
-        self.nal_queue = Queue(3)
-
-    async def write(self, frame, framerate=0):
-        self.framerate = framerate
-        if not self.nal_queue.full():
-            self.nal_queue.put(frame)
-
-    async def run(self):
-        while True:
-            buf = self.nal_queue.get()
-            if buf is not None or buf:
-                await self.onframe(self.gsdbs, self.target, buf, self.framerate)
-
-
 class GSMediaConsumerAsync:
 
-    def __init__(self, gsdbs, target, onframe):
+    def __init__(self, gsdbs, target, onframe, frame_queue):
         self.__tracks = {}
         self.gsdbs = gsdbs
         self.target = target
         self.onframe = onframe
-        self.nal_queue = Queue(3)
-        self.broadcastthread = FrameBufferThreadAsync(self.gsdbs, self.target, self.onframe)
-        loop = asyncio.get_event_loop()
-        self.bthread = loop.create_task(self.startBroadcastTrack())
-        # t2 = threading.Thread(target=asyncio.run, args=(,))
-        # t2.start()
-
-    async def startBroadcastTrack(self):
-        await self.broadcastthread.run()
+        self.frame_queue = frame_queue
 
     def addTrack(self, track):
         if track not in self.__tracks:
@@ -421,10 +331,6 @@ class GSMediaConsumerAsync:
         """
         for track, task in self.__tracks.items():
             if task is None:
-                # self.startProcessing(task)
-                # _thread = threading.Thread(target=self.startProcessing,
-                #                            args=(track,))
-                # _thread.start()
                 self.__tracks[track] = asyncio.ensure_future(
                     self.stream_consume(track, self.gsdbs, self.target, self.onframe))
 
@@ -432,14 +338,11 @@ class GSMediaConsumerAsync:
         while True:
             try:
                 frame = await track.recv()
-                await self.broadcastthread.write(frame)
+                await self.frame_queue.put(frame)
             except MediaStreamError:
                 return
 
     async def stop(self):
-        """
-        Stop discarding media.
-        """
         for task in self.__tracks.values():
             if task is not None:
                 task.cancel()
